function result = ErrorRate_MF0_StreamLevel_Overall(TrkF0s, GTF0s, FreqDevThL, FreqDevThU)
% function result = ErrorRate_MF0_StreamLevel_Overal(TrkF0s, GTF0s, FreqDevThL, FreqDevThU)
% Evaluate multi-F0 estimation and streaming accuracies for all tracks
% collectively. This function automatically tries different orders to match
% the estimated tracks and the ground-truth track.
%
% Input
%   - TrkF0s        : estimated F0 tracks
%   - GTF0s         : ground-truth F0 tracks
%   - FreqDevThL    : frequency deviation lower threshold 
%   - FreqDevThU    : frequency deviation higher threshold
% Output
%   - result        : [MulAcc, MulPre, MulRec] for all the tracks together
%       - MulAcc    : Multiple F0 Accuracy (TP/(TP+FP+FN)), proposed in [1]
%       - MulPre    : Multiple F0 Precision (TP/(TP+FP))
%       - MulRec    : Multiple F0 Recall (TP/(TP+FN))
%
% [1] S. Dixon, "On the computer recognition of solo piano music," in Proc. Australasian Computer Music Conference, Brisbane, 2000.
%
% Author: Zhiyao Duan
% Created: 8/1/2012
% Last modified: 9/10/2016

if nargin < 4
    FreqDevThU = 0.5;
end
if nargin < 3
    FreqDevThL = 0.5;
end

% number of tracks and number of frames
[nC1, nI1] = size(GTF0s);
[nC2, nI2] = size(TrkF0s);

% report error, if they have different number of frames
if nI1 ~= nI2
    error('The two F0 matrices do not match in time!');
end

% fill with zeros, if one F0 matrix has fewer tracks than the
% other
if nC1 < nC2
    GTF0s = [GTF0s; zeros(nC2-nC1, nI1)];
elseif nC2 < nC1
    TrkF0s = [TrkF0s; zeros(nC1-nC2, nI2)];
end

[nC, ~] = size(GTF0s);

% try all possible permutations of track labels when matching the two
% clusterings, and find the best permutation according to the overall
% accuracy
P = perms(1:nC);
permresult = zeros(size(P,1), 3);
for i = 1 : size(P, 1)
    order = P(i, :);
    nCorrect = 0;               % total number of correctly tracked pitches
    nTrans = 0;                 % total number of transcribed pitches
    nRef = 0;                   % total number of reference pitches
    for j = 1:nC
        tempresult = ErrorRate_MF0_FrameLevel(TrkF0s(order(j),:), GTF0s(j,:), FreqDevThL, FreqDevThU);
        nCorrect = nCorrect + sum(GTF0s(j,:)~=0) * tempresult.MulRec;
        nTrans = nTrans + sum(TrkF0s(order(j),:)~=0);
        nRef = nRef + sum(GTF0s(j,:)~=0);
    end
    permresult(i, :) = [nCorrect/(nTrans+nRef-nCorrect), nCorrect/nTrans, nCorrect/nRef];
end
% return the best result of all permutations
[~, idx] = max(permresult(:,1));
result = permresult(idx, :);